from chatminer.chatparsers import InstagramJsonParser, ParsedMessageCollection


def test_instagram():
    target = ParsedMessageCollection()
    target.read_from_json("test/instagram/target.json")
    parser = InstagramJsonParser("test/instagram/test_export.json")
    parser.parse_file()
    assert parser.parsed_messages == target



import calendar
import datetime

import matplotlib.pyplot as plt
import numpy as np
import pandas as pd
from dateutil.relativedelta import relativedelta
from matplotlib.colors import ColorConverter, ListedColormap
from matplotlib.patches import Circle, Polygon, RegularPolygon
from matplotlib.path import Path
from matplotlib.projections import register_projection
from matplotlib.projections.polar import PolarAxes
from matplotlib.spines import Spine
from matplotlib.transforms import Affine2D
from mpl_toolkits.axes_grid1 import make_axes_locatable
from wordcloud import STOPWORDS, WordCloud


def sunburst(
    df: pd.DataFrame,
    color="C0",
    edgecolor="black",
    linewidth=0.5,
    highlight_max=False,
    isolines=None,
    isolines_relative=True,
    ax=None,
    authors=None,
) -> PolarAxes:
    if authors:
        df = df[df["author"].isin(authors)]

    df_circle = df.groupby(by="hour")["message"].count().reset_index()

    hourly_count = np.zeros(24)
    hourly_count[df_circle["hour"]] = df_circle["message"]

    if ax is None:
        _, ax = plt.subplots(subplot_kw={"projection": "polar"})
    assert ax is not None  # For type-checking

    x = np.arange(0, 2 * np.pi, 2 * np.pi / len(hourly_count)) + np.pi / len(
        hourly_count
    )

    alpha = 0.6 if highlight_max else 1
    ax.bar(
        x,
        hourly_count,
        width=2 * np.pi / len(hourly_count),
        alpha=alpha,
        color=color,
        bottom=0,
        edgecolor=edgecolor,
        linewidth=linewidth,
    )

    if highlight_max:
        max_ind = np.argmax(hourly_count)
        ax.bar(
            x[max_ind],
            hourly_count[max_ind],
            bottom=0,
            width=2 * np.pi / len(hourly_count),
            alpha=1,
            color=color,
            edgecolor=edgecolor,
            linewidth=linewidth,
        )

    ax.bar(
        x,
        np.max(hourly_count) * np.ones(len(hourly_count)),
        width=2 * np.pi / len(hourly_count),
        alpha=0.1,
        bottom=0,
        color=color,
    )

    ax.set_theta_direction(-1)
    ax.spines["polar"].set_visible(True)
    ax.set_rmax(np.max(hourly_count))
    ax.grid(True)
    ax.set_axisbelow(True)

    ax.set_theta_offset(np.pi / 2)
    ax.set_xticks(np.linspace(0, 2 * np.pi, 8, endpoint=False))
    ticks = [f"{x}:00" for x in range(0, 24, 3)]
    ax.set_xticklabels(ticks)

    if isolines:
        if isolines_relative:
            ax.set_yticks(np.asarray(isolines) * np.max(hourly_count))
        else:
            ax.set_yticks(isolines)

    plt.tight_layout()
    return ax


def wordcloud(
    df,
    ax=None,
    stopwords=None,
    authors=None,
    **kwargs,
) -> plt.Axes:
    if authors:
        df = df[df["author"].isin(authors)]

    messages = [mess.split() for mess in df["message"].values]
    words = [word.lower() for sublist in messages for word in sublist]

    if stopwords:
        STOPWORDS.update(stopwords)
    stopwords = STOPWORDS

    wc = WordCloud(
        stopwords=stopwords,
        **kwargs,
    )
    wc.generate(" ".join(words))

    if ax is None:
        _, ax = plt.subplots()

    ax.imshow(wc, interpolation="bilinear")
    plt.axis("off")
    plt.tight_layout()
    return ax


def calendar_heatmap(
    df: pd.DataFrame,
    year: int,
    vmin=None,
    vmax=None,
    cmap="Blues",
    fillcolor="whitesmoke",
    linewidth=1,
    linecolor=None,
    daylabels=calendar.day_abbr[:],
    dayticks=True,
    monthly_border=False,
    ax=None,
    authors=None,
    **kwargs,
) -> plt.Axes:
    """
    Adapted from https://github.com/MarvinT/calmap.
    Copyright (c) 2015 by Martijn Vermaat and contributors
    """

    if authors:
        df = df[df["author"].isin(authors)]

    if year not in df["timestamp"].dt.year.values:
        available_years = df["timestamp"].dt.year.unique()
        raise ValueError(
            f"No message in year {year}. Available years: {available_years}"
        )

    df = df[df["timestamp"].dt.year == year]
    df = df.groupby(by=df["timestamp"].dt.date).count()["message"].reset_index()

    idx = pd.date_range(start=str(year), end=str(year + 1), freq="D")[:-1]
    df = df.set_index("timestamp").reindex(idx)

    if vmin is None:
        vmin = int(df["message"].min())
    if vmax is None:
        vmax = int(df["message"].max())

    if ax is None:
        ax = plt.gca()
    assert isinstance(ax, plt.Axes)  # For type-checking

    if linecolor is None:
        linecolor = ax.get_facecolor()
        if ColorConverter().to_rgba(linecolor)[-1] == 0:
            linecolor = "white"

    df["fill"] = 1
    df["day"] = df.index.dayofweek
    df["week"] = df.index.isocalendar().week

    df.loc[(df.index.month == 1) & (df.week > 50), "week"] = 0
    df.loc[(df.index.month == 12) & (df.week < 10), "week"] = df.week.max() + 1

    plot_data = df.pivot(index="day", columns="week", values="message").values[::-1]
    plot_data = np.ma.masked_where(np.isnan(plot_data), plot_data)

    fill_data = df.pivot(index="day", columns="week", values="message").values[::-1]
    fill_data = np.ma.masked_where(np.isnan(fill_data), fill_data)

    ax.pcolormesh(fill_data, vmin=0, vmax=1, cmap=ListedColormap([fillcolor]))

    kwargs["linewidth"] = linewidth
    kwargs["edgecolors"] = linecolor
    pc = ax.pcolormesh(plot_data, vmin=vmin, vmax=vmax, cmap=cmap, **kwargs)

    divider = make_axes_locatable(ax)
    cax: plt.Axes = divider.append_axes("right", size="3%", pad=0.08)
    assert isinstance(cax, plt.Axes)
    plt.colorbar(pc, cax=cax, ticks=[vmin, int((vmin + vmax) / 2), vmax])

    ax.set(xlim=(0, plot_data.shape[1]), ylim=(0, plot_data.shape[0]))

    ax.set_aspect("equal")

    for side in ("top", "right", "left", "bottom"):
        ax.spines[side].set_visible(False)
    ax.xaxis.set_tick_params(which="both", length=0)
    ax.yaxis.set_tick_params(which="both", length=0)

    if dayticks is True:
        dayticks = range(len(daylabels))
    elif dayticks is False:
        dayticks = []
    elif isinstance(dayticks, int):
        dayticks = range(len(daylabels))[dayticks // 2 :: dayticks]

    ax.set_xlabel("")

    xticks, labels = [], []
    for month in range(1, 13):
        first = datetime.datetime(year, month, 1)
        last = first + relativedelta(months=1, days=-1)
        y0 = 6 - first.weekday()
        y1 = 6 - last.weekday()
        start = datetime.datetime(year, 1, 1).weekday()
        x0 = (int(first.strftime("%j")) + start - 1) // 7
        x1 = (int(last.strftime("%j")) + start - 1) // 7
        P = [
            (x0, y0 + 1),
            (x0, 0),
            (x1, 0),
            (x1, y1),
            (x1 + 1, y1),
            (x1 + 1, 7),
            (x0 + 1, 7),
            (x0 + 1, y0 + 1),
        ]

        xticks.append(x0 + (x1 - x0 + 1) / 2)
        labels.append(first.strftime("%b"))
        if monthly_border:
            poly = Polygon(
                P,
                edgecolor="black",
                facecolor="None",
                linewidth=1,
                zorder=20,
                clip_on=False,
            )
            ax.add_artist(poly)

    ax.set_xticks(xticks)
    ax.set_xticklabels(labels)
    ax.set_ylabel("")
    ax.yaxis.set_ticks_position("left")
    ax.set_yticks([6 - i + 0.5 for i in dayticks])
    ax.set_yticklabels(
        [daylabels[i] for i in dayticks], rotation="horizontal", va="center"
    )
    plt.tight_layout()
    return ax


def radar(
    df,
    color="C0",
    alpha=0.3,
    ax=None,
    authors=None,
) -> plt.Axes:
    if authors:
        df = df[df["author"].isin(authors)]

    cats = [
        "Monday",
        "Tuesday",
        "Wednesday",
        "Thursday",
        "Friday",
        "Saturday",
        "Sunday",
    ]
    df_radar = df.groupby(by="weekday")["message"].count().reindex(cats)

    theta = radar_factory(7, frame="polygon")

    if ax is None:
        _, ax = plt.subplots(subplot_kw={"projection": "radar"})

    ax.plot(theta, df_radar.values, color=color)
    ax.fill(theta, df_radar.values, facecolor=color, alpha=alpha)
    ax.set_varlabels(cats)
    return ax


def radar_factory(num_vars, frame="circle"):
    """
    Source: https://matplotlib.org/stable/gallery/specialty_plots/radar_chart.html
    """
    # calculate evenly-spaced axis angles
    theta = np.linspace(0, 2 * np.pi, num_vars, endpoint=False)

    class RadarTransform(PolarAxes.PolarTransform):
        def transform_path_non_affine(self, path: Path):
            # Paths with non-unit interpolation steps correspond to gridlines,
            # in which case we force interpolation (to defeat PolarTransform's
            # autoconversion to circular arcs).
            if path._interpolation_steps > 1:
                path = path.interpolated(num_vars)
            return Path(self.transform(path.vertices), path.codes)

    class RadarAxes(PolarAxes):
        name = "radar"
        PolarTransform = RadarTransform

        def __init__(self, *args, **kwargs):
            super().__init__(*args, **kwargs)
            # rotate plot such that the first axis is at the top
            self.set_theta_zero_location("N")

        def fill(self, *args, closed=True, **kwargs):
            """Override fill so that line is closed by default"""
            return super().fill(closed=closed, *args, **kwargs)

        def plot(self, *args, **kwargs):
            """Override plot so that line is closed by default"""
            lines = super().plot(*args, **kwargs)
            for line in lines:
                self._close_line(line)

        @staticmethod
        def _close_line(line):
            x, y = line.get_data()
            # FIXME: markers at x[0], y[0] get doubled-up
            if x[0] != x[-1]:
                x = np.append(x, x[0])
                y = np.append(y, y[0])
                line.set_data(x, y)

        def set_varlabels(self, labels):
            self.set_thetagrids(np.degrees(theta), labels)

        def _gen_axes_patch(self):
            # The Axes patch must be centered at (0.5, 0.5) and of radius 0.5
            # in axes coordinates.
            if frame == "circle":
                return Circle((0.5, 0.5), 0.5)
            if frame == "polygon":
                return RegularPolygon((0.5, 0.5), num_vars, radius=0.5, edgecolor="k")
            raise ValueError(f"Unknown value for 'frame': {frame}")

        def _gen_axes_spines(self):
            if frame == "circle":
                return super()._gen_axes_spines()
            if frame == "polygon":
                # spine_type must be 'left'/'right'/'top'/'bottom'/'circle'.
                spine = Spine(
                    axes=self,
                    spine_type="circle",
                    path=Path.unit_regular_polygon(num_vars),
                )
                # unit_regular_polygon gives a polygon of radius 1 centered at
                # (0, 0) but we want a polygon of radius 0.5 centered at (0.5,
                # 0.5) in axes coordinates.
                spine.set_transform(
                    Affine2D().scale(0.5).translate(0.5, 0.5) + self.transAxes
                )
                return {"polar": spine}
            raise ValueError(f"Unknown value for 'frame': {frame}")

    register_projection(RadarAxes)
    return theta


def is_radar_registered():
    try:
        plt.subplots(subplot_kw={"projection": "radar"})
    except ValueError:
        return False
    return True



from chatminer.chatparsers import ParsedMessageCollection, WhatsAppParser


def is_equal_to_target(sourcefile):
    targetfile = "test/whatsapp/target.json"
    target = ParsedMessageCollection()
    target.read_from_json(targetfile)
    parser = WhatsAppParser(sourcefile)
    parser.parse_file()
    return parser.parsed_messages == target


def test_mmddyy_24hrs():
    assert is_equal_to_target("test/whatsapp/test_mmddyy_24hrs.txt")


def test_ddmmyy_24hrs():
    assert is_equal_to_target("test/whatsapp/test_ddmmyy_24hrs.txt")


def test_mmddyyyy_12hrs():
    assert is_equal_to_target("test/whatsapp/test_mmddyyyy_12hrs.txt")


def test_yyyymmdd_24hrs():
    assert is_equal_to_target("test/whatsapp/test_yyyymmdd_24hrs.txt")


def test_mmddyy_brackets_24hrs():
    assert is_equal_to_target("test/whatsapp/test_[mmddyy]_24hrs.txt")



from chatminer.chatparsers import ParsedMessageCollection, SignalParser


def test_signal():
    target = ParsedMessageCollection()
    target.read_from_json("test/signal/target.json")
    parser = SignalParser("test/signal/test_export.txt")
    parser.parse_file()
    assert parser.parsed_messages == target



from chatminer.chatparsers import ParsedMessageCollection, TelegramJsonParser


def test_telegram_single_export():
    target = ParsedMessageCollection()
    target.read_from_json("test/telegram/target.json")
    parser = TelegramJsonParser("test/telegram/test_single_export.json")
    parser.parse_file()
    assert parser.parsed_messages == target


def test_telegram_batch_export():
    target = ParsedMessageCollection()
    target.read_from_json("test/telegram/target.json")
    parser = TelegramJsonParser("test/telegram/test_batch_export.json", "Chatname")
    parser.parse_file()
    assert parser.parsed_messages == target



from setuptools import find_packages, setup

setup(
    name="chat-miner",
    description="chat-miner provides lean parsers for every major platform transforming chats into pandas dataframes.\
         Artistic visualizations allow you to explore your data and create artwork from your chats.",
    packages=find_packages(),
    install_requires=[
        "argparse",
    ],
    entry_points={
        "console_scripts": [
            "chatminer = chatminer.cli:main",
        ],
    },
)



import datetime as dt
import json
import logging
import re
import unicodedata
from abc import ABC, abstractmethod
from dataclasses import asdict, dataclass
from pathlib import Path
from typing import Any, Dict, List, Optional

import pandas as pd
from dateutil import parser as datetimeparser
from tqdm import tqdm
from tqdm.contrib.logging import logging_redirect_tqdm

logging.basicConfig(
    level=logging.INFO,
    format="%(asctime)s %(levelname)-8s %(message)s",
    datefmt="%d.%m.%Y %H:%M:%S",
)


@dataclass(frozen=True)
class ParsedMessage:
    timestamp: dt.datetime
    author: str
    message: str


class ParsedMessageCollection:
    def __init__(self):
        self._parsed_messages: List[ParsedMessage] = []

    def append(self, mess: ParsedMessage):
        self._parsed_messages.append(mess)

    def get_df(self):
        messages_as_dict = [asdict(mess) for mess in self._parsed_messages]
        df = pd.DataFrame(messages_as_dict)
        df["weekday"] = df["timestamp"].dt.day_name()
        df["hour"] = df["timestamp"].dt.hour
        df["words"] = df["message"].apply(lambda s: len(s.split(" ")))
        df["letters"] = df["message"].apply(len)
        return df

    def write_to_json(self, file: str):
        def serialize_message(mess: ParsedMessage):
            return {
                "timestamp": mess.timestamp.isoformat(),
                "author": mess.author,
                "message": mess.message,
            }

        with open(file, "w") as json_file:
            json.dump(
                [serialize_message(mess) for mess in self._parsed_messages],
                json_file,
                indent=4,
            )

    def read_from_json(self, file: str):
        def deserialize_message(mess: dict):
            timestamp = dt.datetime.fromisoformat(mess["timestamp"])
            author = mess["author"]
            message = mess["message"]
            return ParsedMessage(timestamp=timestamp, author=author, message=message)

        with open(file, "r") as json_file:
            self._parsed_messages = [
                deserialize_message(mess) for mess in json.load(json_file)
            ]

    def __eq__(self, other):
        return self._parsed_messages == other._parsed_messages


class Parser(ABC):
    def __init__(self, filepath: str):
        self._file = Path(filepath)
        assert self._file.is_file()

        self._raw_messages: List[Dict[str, Any]] | List[str] = []
        self.parsed_messages = ParsedMessageCollection()

        self._logger = logging.getLogger(__name__)
        self._logger.info(
            """
            Depending on the platform, the message format in chat logs might not be
            standardized across devices/versions/localization and might change over
            time. Please report issues including your message format via GitHub.
            """
        )
        self._logger.info("Initialized parser.")

    def parse_file(self):
        self._logger.info("Starting reading raw messages...")
        self._read_raw_messages_from_file()
        self._logger.info("Finished reading %i raw messages.", len(self._raw_messages))

        self._logger.info("Starting parsing raw messages...")
        self._parse_raw_messages()
        self._logger.info("Finished parsing raw messages.")

    @abstractmethod
    def _read_raw_messages_from_file(self):
        ...

    def _parse_raw_messages(self):
        with logging_redirect_tqdm():
            for raw_mess in tqdm(self._raw_messages):
                parsed_mess = self._parse_message(raw_mess)
                if parsed_mess:
                    self.parsed_messages.append(parsed_mess)

    @abstractmethod
    def _parse_message(self, mess: Any) -> Optional[ParsedMessage]:
        ...


class SignalParser(Parser):
    def _read_raw_messages_from_file(self):
        def _is_new_message(line: str):
            regex = r"^\[\d{4}-\d{2}-\d{2}, \d{2}:\d{2}\]"
            return re.match(regex, line)

        with self._file.open(encoding="utf-8") as f:
            lines = reversed(list(f))

        self._raw_messages: List[str]
        buffer: List[str] = []
        for line in lines:
            if not line:
                continue

            if _is_new_message(line):
                if buffer:
                    buffer.append(line)
                    buffer.reverse()
                    joined_buffer = " ".join(buffer)
                    self._raw_messages.append("".join(joined_buffer.splitlines()))
                    buffer.clear()
                else:
                    self._raw_messages.append(line)
            else:
                buffer.append(line)

    def _parse_message(self, mess: str):
        datetime_raw, mess = mess.split("]", 1)
        time = datetimeparser.parse(datetime_raw[1:])
        author, body = mess.split(":", 1)
        author = author.strip()
        body = body.strip()
        return ParsedMessage(time, author, body)


class WhatsAppParser(Parser):
    def __init__(self, filepath: str):
        super().__init__(filepath)
        self._datefmt = WhatsAppDateFormat(self._logger)

    def _read_raw_messages_from_file(self):
        def _is_new_message(line: str):
            regex = r"^[\u200e]?\[?(\d{1,4})([./,-])\d{1,2}\2\d{2,4}([, ])"
            return re.match(regex, line)

        with self._file.open(encoding="utf-8") as f:
            lines = reversed(list(f))

        self._raw_messages: List[str]
        buffer: List[str] = []
        for line in lines:
            if not line:
                continue

            if _is_new_message(line):
                line = line.replace("\u200e", "")
                line = unicodedata.normalize("NFKC", line.strip())
                if buffer:
                    buffer.append(line)
                    buffer.reverse()
                    joined_buffer = " ".join(buffer)
                    self._raw_messages.append("".join(joined_buffer.splitlines()))
                    buffer.clear()
                else:
                    self._raw_messages.append(line)
            else:
                buffer.append(line)

        self._datefmt.infer_format(self._raw_messages)

    def _parse_message(self, mess: str):
        datestr, author_and_body = mess.split(self._datefmt.date_author_sep, 1)
        time = datetimeparser.parse(
            datestr, dayfirst=self._datefmt.is_dayfirst, fuzzy=True
        )

        if ": " in author_and_body:
            author, body = [x.strip() for x in author_and_body.split(": ", 1)]
            return ParsedMessage(time, author, body)
        elif ":." in author_and_body:
            self._logger.info(f"Ignoring self-destroying message on {time}.")
        else:
            body = author_and_body.strip()
            self._logger.info(f"Ignoring sytem message on {time}: {body}.")


class FacebookMessengerParser(Parser):
    def _read_raw_messages_from_file(self):
        with self._file.open(encoding="utf-8") as f:
            self._raw_messages: List[Dict[str, Any]] = json.load(f)["messages"]

    def _parse_message(self, mess: Dict[str, Any]):
        if "content" not in mess:
            return None

        body = mess["content"]

        time = dt.datetime.utcfromtimestamp(mess["timestamp_ms"] / 1000)
        author = mess["sender_name"].encode("latin-1").decode("utf-8")
        body = body.encode("latin-1").decode("utf-8")
        return ParsedMessage(time, author, body)


class InstagramJsonParser(Parser):
    def _read_raw_messages_from_file(self):
        with self._file.open(encoding="utf-8") as f:
            self._raw_messages: List[Dict[str, Any]] = json.load(f)["messages"]

    def _parse_message(self, mess: Dict[str, Any]):
        if "content" not in mess:
            return None

        system_messages = [
            "to your message",
            "in the poll.",
            "created a poll: ",
            "liked a message",
            "This poll is no longer available.",
            "'s poll has multiple updates.",
        ]

        if any(flag in mess["content"] for flag in system_messages):
            return None

        body = mess["content"]

        time = dt.datetime.utcfromtimestamp(mess["timestamp_ms"] / 1000)
        author = mess["sender_name"].encode("latin-1").decode("utf-8")
        body = body.encode("latin-1").decode("utf-8")
        return ParsedMessage(time, author, body)


class TelegramJsonParser(Parser):
    def __init__(self, filepath: str, chat_name: Optional[str] = None):
        super().__init__(filepath)
        self.chat_name: Optional[str] = chat_name

    def _read_raw_messages_from_file(self):
        with self._file.open(encoding="utf-8") as f:
            json_objects = json.load(f)

        if "messages" in json_objects:
            self._logger.info("Detected single chat export.")
            self._raw_messages = json_objects["messages"]
        else:
            self._logger.info("Detected batch export.")
            if self.chat_name:
                for chat in json_objects["chats"]["list"]:
                    if "name" in chat and chat["name"] == self.chat_name:
                        self._raw_messages = chat["messages"]
                        break
            else:
                raise ValueError(f"{self.chat_name} not found in {self._file}")

    def _parse_message(self, mess: Dict[str, Any]):
        if "from" not in mess or "text" not in mess:
            return None

        if isinstance(mess["text"], str):
            body = mess["text"]
        elif isinstance(mess["text"], list):
            text_elements = [
                m["text"] if isinstance(m, dict) else m for m in mess["text"]
            ]
            body = " ".join(text_elements)

        time = dt.datetime.utcfromtimestamp(int(mess["date_unixtime"]))
        author = mess["from"]
        return ParsedMessage(time, author, body)


class WhatsAppDateFormat:
    def __init__(self, logger: logging.Logger):
        self.is_dayfirst = None
        self.is_yearfirst = None
        self.has_brackets = None
        self.date_sep = None
        self.date_author_sep = None
        self._logger = logger

    def infer_format(self, raw_messages: List[str]):
        self.has_brackets = self._infer_brackets(raw_messages[0])
        self.date_author_sep = self._infer_date_author_sep()
        self.date_sep = self._infer_date_sep(raw_messages[0])
        self.is_yearfirst = self._infer_yearfirst(raw_messages[0])
        self.is_dayfirst = self._infer_dayfirst(raw_messages)
        self._log_resulting_format()

    @staticmethod
    def _infer_brackets(mess: str):
        return mess[0] == "["

    def _infer_date_author_sep(self):
        return "]" if self.has_brackets else " - "

    def _infer_date_sep(self, mess: str):
        datestr = mess.split(self.date_author_sep, 1)[0]
        if self.has_brackets:
            datestr = datestr.lstrip("[").rstrip("]")
        for c in datestr:
            if not c.isdigit():
                return c
        raise ValueError("No non-numeric character in datestring.")

    def _infer_yearfirst(self, mess: str):
        datestr = mess.split(self.date_author_sep, 1)[0]
        if self.has_brackets:
            datestr = datestr.lstrip("[").rstrip("]")
        return int(datestr.split(self.date_sep)[0]) >= 100

    def _infer_dayfirst(self, raw_messages: List[str]):
        max_first = 0
        max_second = 0
        for mess in raw_messages:
            datestr = mess.split(self.date_author_sep, 1)[0]

            if self.has_brackets:
                datestr = datestr.lstrip("[").rstrip("]")

            if self.is_yearfirst:
                datepart = datestr.split(",")[0]
                day_and_month = [int(num) for num in datepart.split(self.date_sep)[1:3]]
            else:
                day_and_month = [int(num) for num in datestr.split(self.date_sep)[:2]]

            max_first = max(max_first, day_and_month[0])
            max_second = max(max_second, day_and_month[1])

            if max_first > 12 or max_second > 12:
                break

        if max_first > 12 >= max_second:
            return True
        if max_first <= 12 < max_second:
            return False
        self._logger.warning(
            "Can't infer date format: No day > 12. Falling back on day first."
        )
        return True

    def _log_resulting_format(self):
        start = "[" if self.has_brackets else ""
        end = "]" if self.has_brackets else ""
        if self.is_yearfirst:
            date1 = "year"
            date2 = "month"
            date3 = "day"
        elif self.is_dayfirst:
            date1 = "day"
            date2 = "month"
            date3 = "year"
        else:
            date1 = "month"
            date2 = "day"
            date3 = "year"
        composition = f"{start}{date1}{self.date_sep}{date2}{self.date_sep}{date3}{end}"
        self._logger.info("Inferred date format: %s", composition)
